#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "util.h"
#include "ranlib.h"
#include "header.h"

int main(int argc, char *argv[]){
  int t,u,p,points,r;
  long seed1, seed2;
  double vih, sum=0, sumsquared=0, *sample, *avg_without, jerror, jbias, average, stddev;
  int i1,i,j;
  time_t now;

  now = time(0);
  phrtsd(PHRASE,&seed1, &seed2);
  setall(SEED1, SEED2);
  
  if(argc != 15){
    printf("Usage: %s UNDERLIERS TIMESLICES STRIKE INITIAL EXPIRATION INTEREST\n",argv[0]);
    printf("DIVIDEND VARIANCE POINTS MINPOINTS POWER PROBCUTOFF MAXIGNORE REPETITIONS\n");
    exit(1);
  }

  sscanf(argv[1],"%i",&UNDERLIERS);
  sscanf(argv[2],"%i",&TIMESLICES);
  sscanf(argv[3],"%lf",&STRIKE);
  sscanf(argv[4],"%lf",&INITIAL);
  sscanf(argv[5],"%lf",&EXPIRATION);
  sscanf(argv[6],"%lf",&INTEREST); INTEREST = INTEREST*EXPIRATION/(double)(TIMESLICES-1.0);
  sscanf(argv[7],"%lf",&DIVIDEND); DIVIDEND = DIVIDEND*EXPIRATION/(double)(TIMESLICES-1.0);
  sscanf(argv[8],"%lf",&VARIANCE); VARIANCE = VARIANCE*EXPIRATION/(double)(TIMESLICES-1.0);
  sscanf(argv[9],"%i",&POINTS);
  sscanf(argv[10],"%i",&MINPOINTS);
  sscanf(argv[11],"%lf",&POWER);
  sscanf(argv[12],"%lf",&PROBCUTOFF);
  sscanf(argv[13],"%i",&MAXIGNORE);
  sscanf(argv[14],"%i",&REPETITIONS);
  
  printf("Started     : %s\n",asctime(localtime(&now)));
  printf("UNDERLIERS  : %i (%i max)\n",UNDERLIERS, UNDERLIERS_SIZE);
  printf("TIMESLICES  : %i (%i max)\n",TIMESLICES, TIMESLICES_SIZE);
  printf("STRIKE      : %lf\n",STRIKE);
  printf("INITIAL     : %lf\n",INITIAL);
  printf("EXPIRATION  : %lf\n",EXPIRATION);
  printf("INTERSET    : %lf per period, %lf per timeslice\n",(double)(TIMESLICES-1.0)*INTEREST/EXPIRATION,INTEREST);
  printf("DIVIDEND    : %lf per period, %lf per timeslice\n",(double)(TIMESLICES-1.0)*DIVIDEND/EXPIRATION,DIVIDEND);
  printf("VARIANCE    : %lf per period, %lf per timeslice\n",(double)(TIMESLICES-1.0)*VARIANCE/EXPIRATION,VARIANCE);
  printf("POINTS      : %i (%i max)\n",POINTS,POINTS_SIZE);
  printf("MINPOINTS   : %i\n",MINPOINTS);
  printf("POWER       : %lf\n",POWER);
  printf("PROBCUTOFF  : %lf\n",PROBCUTOFF);
  printf("MAXIGNORE   : %i\n",MAXIGNORE);
  printf("REPETITIONS : %i\n",REPETITIONS);
  
      
  sample = (double*)malloc(REPETITIONS*sizeof(double));
  avg_without = (double*)malloc(REPETITIONS*sizeof(double));

  for(r=0;r<REPETITIONS;++r){
    for(u=0;u<UNDERLIERS;++u){
      Price[0][u][0] = INITIAL;
      Prob[0][u][0] = 1.0/UNDERLIERS;
    }

    Max1[0][0] = 0;
    Max2[0][0] = 1;

    if(INITIAL>STRIKE){
      Value_if_exercised[0][0] = INITIAL-STRIKE;
    }else{
      Value_if_exercised[0][0] = 0;
    }

    /* Fill Price[][][], Max1[][], Max2[][], Value_if_exercised[][], last timeslice Value[][] */
    for(t=1;t<TIMESLICES;++t){
      Ignore[t] = 0;
      Ignore_den[t] = 0;
      init_rng(t);
      points = points_on_timeslice(t);
      for(p=0;p<points;++p){
	generate_point(t,p);
      }
    }

    for(p=0;p<POINTS;++p){
      Value[TIMESLICES-1][p] = Value_if_exercised[TIMESLICES-1][p];
    }





    /* DEBUG */
#if DEBUG
    for(t=0;t<TIMESLICES;++t){
      points=points_on_timeslice(t);

      for(p=0;p<points;++p){
	printf("Price[%i][:][%i]\t",t,p);

	printf("(%6.2f",Price[t][0][p]);
	for(u=1;u<UNDERLIERS;++u){
	  printf(", %6.2f",Price[t][u][p]);
	}
	printf(")\t");

	printf("%i\t",Max1[t][p]);
	printf("%i\t",Max2[t][p]);
	printf("%f\n",Value_if_exercised[t][p]);
      }
    }
#endif

    /* Fill Prob[][][] */
    /*   if(PROBCUTOFF){ */
    /*     for(t=1;t<TIMESLICES-1;++t){ */
    /*       points = points_on_timeslice(t); */
    /*       for(p=0;p<points;++p){ */
    /* 	prob_of_max(t,p); */
    /*       } */
    /*     } */
    /*   }else{ */
    /*     for(t=1;t<TIMESLICES-1;++t){ */
    /*       points = points_on_timeslice(t); */
    /*       for(u=0;u<UNDERLIERS;++u){ */
    /* 	for(p=0;p<points;++p){ */
    /* 	  Prob[t][u][p] = 1.0/(double)UNDERLIERS; */
    /* 	} */
    /*       } */
    /*     }     */
    /*   } */


    /* DEBUG */
#if DEBUG
    for(t=0;t<TIMESLICES;++t){
      points = points_on_timeslice(t);
      for(p=0;p<points;++p){
	printf("Prob [%i][:][%i](%15g",t,p,Prob[t][0][p]);
	for(u=1;u<UNDERLIERS;++u){
	  printf(", %15g",Prob[t][u][p]);
	}
	printf(")\n");
      }
      printf("\n");
    }
#endif

    /* Fill Value[][] */
    for(t=TIMESLICES-2;t>0;--t){
      points = points_on_timeslice(t);
      for(p=0;p<points;++p){
	/* Set all Prob[][]'s to 1.0 so at first we don't throw away any underliers */
	for(u=0;u<UNDERLIERS;++u){
	  Prob[t][u][p] = 1.0;
	}
	/* Initial mesh estimate - calculates real Prob[][]'s */
	vih = mesh_estimate_3(t,p);

	/* If We want to throw away any underliers (i1 = how many to throw away) */
	/* because of low Prob[][]'s, we need to call mesh_estimate again. */
	if(i1 = generate_mask(t,p) > 0){
	  vih = mesh_estimate_3(t,p);
	}
	if(vih > Value_if_exercised[t][p]){
	  Value[t][p] = vih;
	}else{
	  Value[t][p] = Value_if_exercised[t][p];
	}
      }
    }
    vih = mesh_estimate_3(0,0);
  
    if(vih > Value_if_exercised[0][0]){
      Value[0][0] = vih;
    }else{
      Value[0][0] = Value_if_exercised[0][0];
    }

    sum += Value[0][0];
    sumsquared += Value[0][0]*Value[0][0];
    sample[r] = Value[0][0];

    /* DEBUG */
#if DEBUG
    for(t=0;t<TIMESLICES;++t){
      points=points_on_timeslice(t);

      for(p=0;p<points;++p){
	printf("Value[%i][%i] = %f\n",t,p,Value[t][p]);
      }
    }
#endif

    printf("%f\t",Value[0][0]);

    for(t=0;t<TIMESLICES-1;++t){
      printf("%f ",Ignore[t]/Ignore_den[t]);
    }
    printf("\n");
  }

  average = sum/(double)REPETITIONS;
  stddev = 1.0/sqrt((double)REPETITIONS-1)* sqrt( (sumsquared/(double)REPETITIONS) - (sum/(double)REPETITIONS)*(sum/(double)REPETITIONS));
  
  printf("Value = %lf +/- %lf\n",sum/(double)REPETITIONS,1.0/sqrt((double)REPETITIONS-1)* sqrt( (sumsquared/(double)REPETITIONS) - (sum/(double)REPETITIONS)*(sum/(double)REPETITIONS)));

/*   for(i=i;i<REPETITIONS;++i){ */
/*     avg_without[i] = 0; */
/*     for(j=0;j<REPETITIONS;++j){ */
/*       if(i != j) avg_without[i] += sample[j]; */
/*     } */
/*     avg_without[i] /= (REPETITIONS-1); */
/*   } */

/*   jerror = 0; */
/*   for(i=0;i<REPETITIONS;++i){ */
/*     jerror += (average - avg_without[i])*(average - avg_without[i]); */
/*   } */
/*   jerror = sqrt( ((double)REPETITIONS-1.0)/(double)REPETITIONS*jerror); */

/*   jbias = 0; */
/*   for(i=0;i<REPETITIONS;++i){ */
/*     jbias += avg_without[i]; */
/*   } */
/*   jbias /= REPETITIONS; */

/*   jbias = ((double)REPETITIONS-1.0)*(jbias-average); */

/*   printf("jerror = %lf\n",jerror); */
/*   printf("jbias = %lf\n",jbias); */

}

int points_on_timeslice(int t){
  int c;

  c = (POINTS-MINPOINTS)/pow(EXPIRATION,POWER);
  return(MINPOINTS + c*pow(EXPIRATION*(double)t/(TIMESLICES-1),POWER));
}

 
